## $Id: export.py 12315 2014-12-24 16:00:58Z henrik $
##
## Copyright (C) 2012 Uli Fouquet & Henrik Bettermann
## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.
##
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with this program; if not, write to the Free Software
## Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
##
"""Exporters for products.
"""
import grok
from zope.securitypolicy.interfaces import IPrincipalRoleMap
from waeup.ikoba.interfaces import ICSVExporter
from waeup.ikoba.interfaces import MessageFactory as _
from waeup.ikoba.utils.batching import ExporterBase
from waeup.ikoba.utils.helpers import iface_names
from waeup.ikoba.products.interfaces import IProduct


class ProductExporter(grok.GlobalUtility, ExporterBase):
    """Exporter for products.
    """
    grok.implements(ICSVExporter)
    grok.name('products')

    #: Fieldnames considered by this exporter
    fields = tuple(sorted(iface_names(IProduct))) + (
        'users_with_local_roles',)

    #: The title under which this exporter will be displayed
    title = _(u'Products')

    def mangle_value(self, value, name, context=None):
        """Hook for mangling values in derived classes
        """
        if name == 'users_with_local_roles':
            value = []
            role_map = IPrincipalRoleMap(context)
            for local_role, user_name, setting in role_map.getPrincipalsAndRoles():
                value.append({'user_name':user_name,'local_role':local_role})
        if name == 'options' and value is not None:
            value = [eval(entry.to_string()) for entry in value]
        return super(ProductExporter, self).mangle_value(
            value, name, context)

    def export(self, products, filepath=None):
        """Export `products`, an iterable, as CSV file.

        If `filepath` is ``None``, a raw string with CSV data is returned.
        """
        writer, outfile = self.get_csv_writer(filepath)
        for product in products:
            self.write_item(product, writer)
        return self.close_outfile(filepath, outfile)

    def export_all(self, site, filepath=None):
        """Export products in productscontainer into filepath as CSV data.

        If `filepath` is ``None``, a raw string with CSV data is returned.
        """
        products = site.get('products', {})
        return self.export(products.values(), filepath)
